"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_codegen_visitor_plugin_common_1 = require("graphql-codegen-visitor-plugin-common");
var autoBind = require("auto-bind");
var graphql_codegen_core_1 = require("graphql-codegen-core");
var ReactApolloVisitor = /** @class */ (function (_super) {
    __extends(ReactApolloVisitor, _super);
    function ReactApolloVisitor(fragments, rawConfig) {
        var _this = _super.call(this, fragments, rawConfig, {
            withHOC: graphql_codegen_visitor_plugin_common_1.getConfigValue(rawConfig.withHOC, true),
            withComponent: graphql_codegen_visitor_plugin_common_1.getConfigValue(rawConfig.withComponent, true),
            withHooks: graphql_codegen_visitor_plugin_common_1.getConfigValue(rawConfig.withHooks, false),
            hooksImportFrom: graphql_codegen_visitor_plugin_common_1.getConfigValue(rawConfig.hooksImportFrom, 'react-apollo-hooks')
        }) || this;
        autoBind(_this);
        return _this;
    }
    ReactApolloVisitor.prototype.getImports = function () {
        var baseImports = _super.prototype.getImports.call(this);
        var imports = [];
        if (this.config.withComponent) {
            imports.push("import * as React from 'react';");
        }
        if (this.config.withComponent || this.config.withHOC) {
            imports.push("import * as ReactApollo from 'react-apollo';");
        }
        if (this.config.withHooks) {
            imports.push("import * as ReactApolloHooks from '" + (typeof this.config.hooksImportFrom === 'string' ? this.config.hooksImportFrom : 'react-apollo-hooks') + "';");
        }
        return [baseImports].concat(imports).join('\n');
    };
    ReactApolloVisitor.prototype._buildHocProps = function (operationName, operationType) {
        var typeVariableName = this.convertName(operationName + graphql_codegen_core_1.toPascalCase(operationType));
        var variablesVarName = this.convertName(operationName + graphql_codegen_core_1.toPascalCase(operationType) + 'Variables');
        var argType = operationType === 'mutation' ? 'MutateProps' : 'DataProps';
        return "Partial<ReactApollo." + argType + "<" + typeVariableName + ", " + variablesVarName + ">>";
    };
    ReactApolloVisitor.prototype._buildOperationHoc = function (node, documentVariableName, operationResultType, operationVariablesTypes) {
        var operationName = this.convertName(node.name.value);
        var propsTypeName = operationName + 'Props';
        var propsVar = "export type " + propsTypeName + "<TChildProps = any> = " + this._buildHocProps(node.name.value, node.operation) + " & TChildProps;";
        var mutationFn = node.operation === 'mutation'
            ? "export type " + this.convertName(node.name.value + 'MutationFn') + " = ReactApollo.MutationFn<" + operationResultType + ", " + operationVariablesTypes + ">;"
            : null;
        var hocString = "export function " + operationName + "HOC<TProps, TChildProps = any>(operationOptions: ReactApollo.OperationOption<\n  TProps, \n  " + operationResultType + ",\n  " + operationVariablesTypes + ",\n  " + propsTypeName + "<TChildProps>> | undefined) {\n    return ReactApollo.graphql<TProps, " + operationResultType + ", " + operationVariablesTypes + ", " + propsTypeName + "<TChildProps>>(" + documentVariableName + ", operationOptions);\n};";
        return [propsVar, mutationFn, hocString].filter(function (a) { return a; }).join('\n');
    };
    ReactApolloVisitor.prototype._buildComponent = function (node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        var componentName = this.convertName(node.name.value + 'Component');
        return "\nexport class " + componentName + " extends React.Component<Partial<ReactApollo." + operationType + "Props<" + operationResultType + ", " + operationVariablesTypes + ">>> {\n  render() {\n      return (\n          <ReactApollo." + operationType + "<" + operationResultType + ", " + operationVariablesTypes + "> " + node.operation + "={" + documentVariableName + "} {...(this as any)['props'] as any} />\n      );\n  }\n}";
    };
    ReactApolloVisitor.prototype._buildHooks = function (node, operationType, documentVariableName, operationResultType, operationVariablesTypes) {
        return "\nexport function use" + operationResultType + "(baseOptions?: ReactApolloHooks." + operationType + "HookOptions<" + (node.operation !== 'query' ? operationResultType + ", " : '') + operationVariablesTypes + ">) {\n  return ReactApolloHooks.use" + operationType + "<" + operationResultType + ", " + operationVariablesTypes + ">(" + documentVariableName + ", baseOptions);\n};";
    };
    ReactApolloVisitor.prototype.buildOperation = function (node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        var component = this.config.withComponent
            ? this._buildComponent(node, documentVariableName, operationType, operationResultType, operationVariablesTypes)
            : null;
        var hoc = this.config.withHOC
            ? this._buildOperationHoc(node, documentVariableName, operationResultType, operationVariablesTypes)
            : null;
        var hooks = this.config.withHooks
            ? this._buildHooks(node, operationType, documentVariableName, operationResultType, operationVariablesTypes)
            : null;
        return [component, hoc, hooks].filter(function (a) { return a; }).join('\n');
    };
    return ReactApolloVisitor;
}(graphql_codegen_visitor_plugin_common_1.ClientSideBaseVisitor));
exports.ReactApolloVisitor = ReactApolloVisitor;
//# sourceMappingURL=visitor.js.map